// Vertex Shader
#version 330 core

in vec3 in_position;
in vec3 in_normal;
in vec2 in_uv;
in vec3 in_tangent;
in vec3 in_bitangent;

//--UNIFORM--//

uniform mat4 M;//model matrix
uniform mat4 V;//view matrix
uniform mat4 P;//projection matrix

uniform vec3 camera_position;

out vec2 uv;
out vec3 E_w;
out vec3 T;
out vec3 B;
out vec3 N;
out vec3 pos_m;


void main()
{

	mat4 MV = V * M;
	mat3 MV3x3 = mat3(MV);
	//Normal/Tangent/Bitangent in camera space // Only correct for uniform scaling in modelmatrix ! Use its inverse transpose if not.
	vec3 N_cam = MV3x3 * normalize(in_normal);
	vec3 T_cam = MV3x3 * normalize(in_tangent);
	vec3 B_cam = MV3x3 * normalize(in_bitangent);

	//tangentspace -> modelspace
	T = normalize(in_tangent);
	B = normalize(in_bitangent);
	N = normalize(in_normal);

	//vertex position in worldspace
	vec3 position_w = (M * vec4(in_position,1)).xyz;

	//eye vec in world space 
	E_w = camera_position - position_w;

	//Forward UV
	uv = in_uv;

	//Forward modelspace vertex position
	pos_m = in_position;

	//Apply MVP
	gl_Position =  P * MV * vec4(in_position, 1);
} 