#version 330 core
uniform sampler2D renderedTexture;
uniform sampler2D depthBuffer;

// hacky use of mvp shader
// V nad P of previous frame
uniform mat4 V;
uniform mat4 P;
// inverse of VP of current frame
uniform mat4 invVP;
uniform float fps;

in vec2 UV;

out vec4 out_color;


void main(){
	vec2 texCoords =UV;
	
	float numSamples = 16;
   // Get the depth buffer value at this pixel.
   float zOverW = texture(depthBuffer, UV).r;
   vec3 color = vec3(zOverW);  

   // H is the viewport position at this pixel in the range -1 to 1.
   vec4 H = vec4(UV.x * 2 - 1, (1 - UV.y) * 2 - 1, zOverW, 1);
   
   // Transform by the view-projection inverse.
   vec4 D = invVP* H ;
	// Divide by w to get the world position.
   vec4 worldPos = D / D.w;
   
	 // Current viewport position
   vec4 currentPos = H;
	// Use the world position, and transform by the previous view-

   
   vec4 previousPos =( P*V) * worldPos;
	// Convert to nonhomogeneous points [-1,1] by dividing by w.
	previousPos /= previousPos.w;
	// Use this frame's position and last frame's to compute the pixel velocity.
	float speed = length(currentPos-previousPos);
	// blur 2000 is not enough
   vec2 velocity = normalize((currentPos - previousPos).xy)*speed*fps/4000.0f;
	
	   // Get the initial color at this pixel.
   vec3 acum = texture(renderedTexture, texCoords).rgb;
	//texCoords = texCoords + velocity;
	for(int i = 1; i < numSamples; ++i)
	{
	  // Sample the color buffer along the velocity vector.
	  // get offset in range [-0.5, 0.5]:
	   vec2 offset = velocity * (float(i) / float(numSamples - 1) - 0.5);
		
		// Add the current color to our color sum. 
	   acum +=  texture(renderedTexture, texCoords+offset).rgb;
	}
	// Average all of the samples to get the final blur color.
	acum = acum / numSamples;
	color = acum;
   
	out_color = vec4(color,1);

}