// Fragment Shader

#version 330 core

layout (location=0) out vec4 fragColor;

//Global alpha
//uniform float alpha;

//From vertex shader in tangentspace
in vec3 L_directional_ts[3]; //= directions
in vec3 L_point_direction_ts[3]; //= direction to point light
in float L_point_attenuation[3]; //= distance to point light
in vec3 E_ts; //view/eye vector

//Texcoords from vertex shader
in vec2 uv;

//Color and Material
uniform float ka; //ambient const.
uniform vec3 cAmbient; //ambient color
uniform float kd; //diffuse
uniform vec3 cDiffuse;
uniform float ks; //specular
uniform vec3 cSpecular;
uniform uint phongPower; //shininess
uniform sampler2D texture_bump;

//Light colors
uniform vec3 cDirectional[3];
uniform vec3 cPoint[3];

void main(void)
{
	//Normalize eye dir after interpolation
	vec3 E = normalize(E_ts);
	
	//vec3 texcolor = texture2D ( color_texture, tex ).rgb; //TODO
	vec3 color = vec3(1);

	//look up bump map normal

	vec3 N = normalize( texture2D(texture_bump, uv).rgb * 2.0 - 1.0 );

	vec3 sum_directional = vec3(0); //sum of directional lights
	vec3 sum_point = vec3(0); //sum of point lights

	//Ambient
	vec3 ambient = ka * cAmbient * color;

	//Directional Lights
	for(int i = 0; i < 3; i++) { 
		vec3 L = normalize(L_directional_ts[i]);
		if (length(cDirectional[i]) != 0) {//check empty light
			//Diffuse (Lambert)
			float cosTheta = clamp( dot(N, L), 0,1);
			vec3 diffuse = kd * cDiffuse * cosTheta;

			//Specular (Phong)
			vec3 R = - reflect(L, N); //minus, weil anders als in cg folien berechnet
			float cosSigma = clamp( dot(E, R), 0,1); 
			vec3 specular = ks * cSpecular * pow(cosSigma,phongPower);

			sum_directional = sum_directional + (diffuse + specular) * cDirectional[i];
		}
	}

	//Point Lights
	for(int i = 0; i < 3; i++) {
		vec3 L = normalize(L_point_direction_ts[i]);
		if (length(cPoint[i]) != 0) {//check empty light
			//Diffuse (Lambert)
			float cosTheta = clamp( dot(N,L), 0,1);
			vec3 diffuse = kd * cDiffuse * cosTheta;

			//Specular (Phong)
			vec3 R = - reflect(L, N);
			float cosSigma = clamp( dot(E, R), 0,1); 
			vec3 specular = ks * cSpecular * pow(cosSigma,phongPower);

			//Accumulate //divided by distance
			sum_point = sum_point + ((diffuse + specular) * cPoint[i]) / (L_point_attenuation[i]);
		}
	}

	//Mix light color and object color
	vec3 total_light = (ambient + sum_directional + sum_point) * color;


	//Add components together
	fragColor = vec4(total_light, 1.0); //TODO ? * alpha);


}
