// Vertex Shader

#version 330 core

//--IN--//

in vec3 in_position;
in vec3 in_normal;
in vec2 in_uv;
//in color
in vec3 in_tangent;
in vec3 in_bitangent;

//--UNIFORM--//

uniform mat4 M;//model matrix
uniform mat4 V;//view matrix
uniform mat4 P;//projection matrix

//Direction of the -single- light in world space //TODO impl. lights
uniform vec3 lights_directional[3];
uniform vec3 lights_point[3];

//--OUT--//

//ts.. tangentspace
out vec3 L_directional_ts[3]; //= directions
out vec3 L_point_direction_ts[3]; //= direction to point light
out float L_point_attenuation[3]; //= distance to point light
out vec3 E_ts; //view/eye vector


out vec2 uv;


void main(void)
{		

	mat4 MV = V * M;
	mat3 MV3x3 = mat3(MV);

	//Normal/Tangent/Bitangent in camera space
	vec3 N_cam = MV3x3 * normalize(in_normal);// Only correct for uniform scaling in modelmatrix ! Use its inverse transpose if not.
	vec3 T_cam = MV3x3 * normalize(in_tangent);
	vec3 B_cam = MV3x3 * normalize(in_bitangent);

	//camspace -> tangentspace matrix
	mat3 TBN = transpose(mat3(T_cam,B_cam,N_cam));


	//vertex position in cameraspace
	vec3 position_cam = (MV * vec4(in_position,1)).xyz;

	//Direction to light
	//Directional Lights
	for (int i = 0; i < 3; i++) { 
		L_directional_ts[i] = TBN * normalize((V * vec4(lights_directional[i], 0)).xyz);
	}
	//Point Lights
	for (int i = 0; i < 3; i++) { 
		vec3 lightpos_cam = (V * vec4(lights_point[i], 1)).xyz;
		vec3 vertexToLight_cam = lightpos_cam - position_cam;
		float distanceToLight = length(vertexToLight_cam);
		L_point_attenuation[i] = distanceToLight * distanceToLight; //Possibly: a * d^2 + b * d + c
		L_point_direction_ts[i] = TBN * normalize((vertexToLight_cam / distanceToLight));
	}
	

	//Direction to camera from vertex, camera in cameraspace is at 0,0,0
	vec3 E_cam = vec3(0,0,0) - position_cam;
	E_ts = TBN * normalize(E_cam);



	//Forward UV
	uv = in_uv;

	//Apply MVP
	gl_Position =  P * MV * vec4(in_position, 1);
}
